
using System;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

namespace TCPIPServer
{
    public partial class Server : Form
    {
        
        TcpListener mTCPListener;  TcpClient mTCPClient; byte[] mRx;

        public Server()
        {
            InitializeComponent();
            tbConsoleOutput.TabStop = false;
        }
        IPAddress findMyIPV4Address()
        {
            string strThisHostName = string.Empty;
            IPHostEntry thisHostDNSEntry = null;
            IPAddress[] allIPsOfThisHost = null;
            IPAddress ipv4Ret = null;

            try
            {strThisHostName = System.Net.Dns.GetHostName();
               printLine("Host Name: " + strThisHostName);
                
                thisHostDNSEntry = System.Net.Dns.GetHostEntry(strThisHostName);
                allIPsOfThisHost = thisHostDNSEntry.AddressList;
                for(int idx = allIPsOfThisHost.Length - 1; idx > 0; idx--)
                {
                    if (allIPsOfThisHost[idx].AddressFamily == AddressFamily.InterNetwork)
                    {
                        ipv4Ret = allIPsOfThisHost[idx];
                        break;
                    }
                }

            } catch (Exception exc)
            {
                MessageBox.Show("Could not find your IPv4 Address, please manually enter and press Listen", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            return ipv4Ret;
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            IPAddress Ipa = null;
            Ipa = findMyIPV4Address();
            if (Ipa != null)
            {
                tbIPAddress.Text = Ipa.ToString();
               
            }
            btnClose.TabStop = false;
            btnClose.FlatAppearance.BorderSize = 0;
            btnClose.FlatStyle = FlatStyle.Flat;
        }
        protected override void WndProc(ref Message m)
        {
            base.WndProc(ref m);
            if (m.Msg == WM_NCHITTEST)
                m.Result = (IntPtr)(HT_CAPTION);
        }

        private const int WM_NCHITTEST = 0x84;
        private const int HT_CLIENT = 0x1;
        private const int HT_CAPTION = 0x2;
        private void btnStartListening_Click(object sender, EventArgs e)
        {
            
            IPAddress ipaddr;
            int nPort;

           
            if (!int.TryParse(tbPort.Text, out nPort))
            {
               
                nPort = 23000;
                tbPort.Text = Convert.ToString(nPort);
            }

            if (!IPAddress.TryParse(tbIPAddress.Text, out ipaddr))
            {
                MessageBox.Show("Invalid IPv4 address supplied.", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                mTCPListener = new TcpListener(ipaddr, nPort);
                mTCPListener.Start();
                mTCPListener.BeginAcceptTcpClient(onCompleteAcceptTcpClient, mTCPListener);
                msgControls.Enabled = true;
                configurationGroup.Enabled = false;
                tbPayLoad.Select();

                printLine("Server is listening and waiting for client to connect...");

            } catch (Exception exc)
            {
                MessageBox.Show("Invalid IPv4 Or Local Host address supplied!", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Restart();
            }

        }
        void onCompleteAcceptTcpClient(IAsyncResult iar)
        {
            byte[] tx = new byte[512];
            TcpListener tcpl = (TcpListener)iar.AsyncState;
            try
            {    
                mTCPClient = tcpl.EndAcceptTcpClient(iar);
                DateTime dateTime = DateTime.Now;

                tx = Encoding.ASCII.GetBytes("You are now connected to the server. You are listening to: " + tbIPAddress.Text +
                    ":" + tbPort.Text + ". \r\n" + "Chat started on: " + dateTime.ToString() + Environment.NewLine);
                printLine("\r\nChat started on: " + dateTime.ToString());
                mTCPClient.GetStream().BeginWrite(tx, 0, tx.Length, onCompleteWriteToClientStream, mTCPClient);
                tcpl.BeginAcceptTcpClient(onCompleteAcceptTcpClient, tcpl);
                mRx = new byte[512];
                mTCPClient.GetStream().BeginRead(mRx, 0, mRx.Length, onCompleteReadFromTCPClientStream, mTCPClient);

            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        void onCompleteReadFromTCPClientStream(IAsyncResult iar)
        {
            //Open a TCPClient connection
            TcpClient tcpc;

            //Store nothing in the Read Bytes
            int nCountReadBytes = 0;

           
            string strRecv;

            try
            {
                tcpc = (TcpClient) iar.AsyncState;
                nCountReadBytes = tcpc.GetStream().EndRead(iar);
                
                if (nCountReadBytes <= 0)
                {
                    printLine("Client has been disconnected. Idle for too long...");
                    printLine("__________________________");
                    return;
                }
                strRecv = Encoding.ASCII.GetString(mRx, 0, nCountReadBytes);
                printLine(strRecv);
                mRx = new byte[512];
                tcpc.GetStream().BeginRead(mRx, 0, mRx.Length, onCompleteReadFromTCPClientStream, tcpc);

            }
            catch (Exception ex)
            {
                MessageBox.Show("A Client has Disconnected!", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Information);
                MessageBox.Show("Server is still listening for incoming connections...", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Information);
                printLine("__________________________");
            }
        }
        private void btnSend_Click(object sender, EventArgs e)
        {
            byte[] tx = new byte[512];
            if (string.IsNullOrEmpty(tbPayLoad.Text))
            {
                MessageBox.Show("There is nothing to send as your message box is empty!", "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            try
            {
                if (mTCPClient != null)
                {
                    if (mTCPClient.Client.Connected) {
                        tx = Encoding.ASCII.GetBytes("Server" + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + tbPayLoad.Text);
                        printLine("Server" + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + tbPayLoad.Text);
                        tbPayLoad.Clear();
                        mTCPClient.GetStream().BeginWrite(tx, 0, tx.Length, onCompleteWriteToClientStream, mTCPClient);

                    }
                }

            } catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        private void onCompleteWriteToClientStream(IAsyncResult iar)
        {
           
            try
            {
                TcpClient tcpc = (TcpClient)iar.AsyncState;
                tcpc.GetStream().EndWrite(iar);

            } catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void btnClose_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        public void printLine(string _strPrint)
        {
            tbConsoleOutput.Invoke(new Action<string>(doInvoke), _strPrint);
        }

        public void doInvoke(string _strPrint)
        {
            tbConsoleOutput.Text = tbConsoleOutput.Text + _strPrint + Environment.NewLine;
        }
    }
}
